"""Utils for the API.
"""
from enum import Enum

import httpx
import requests
from tenacity import retry, retry_if_result, stop_after_attempt, wait_random_exponential


class EmbeddingModel(Enum):
    """Supported embedding models.
    """
    BGE_LARGE_ENG = 'bge-large-en'


class CompletionModel(Enum):
    """Supported completion models.
    """
    MPT_7B_INSTRUCT = 'mpt-7b-instruct'
    MPT_30B_INSTRUCT = 'mpt-30b-instruct'


class ChatCompletionModel(Enum):
    """Supported chat completion models.
    """
    LLAMA_2_70B_CHAT = 'llama-2-70b-chat'
    MIXTRAL_8X7B_CHAT = 'mixtral-8x7b-instruct'


def send_request(client: requests.Session, url, headers, json, timeout):
    if client:
        return client.post(url=url, headers=headers, json=json, timeout=timeout)
    else:
        return requests.post(url=url, headers=headers, json=json, timeout=timeout)


async def asend_request(client: httpx.AsyncClient, url, headers, json, timeout):
    return await client.post(url=url, headers=headers, json=json, timeout=timeout)


def is_internal_server_error(response: requests.Response):
    return response.status_code >= 500



"""Foundation Model API Resource.
"""
import asyncio
import json as json_lib
import os
from typing import Optional

import httpx
import requests
from databricks.sdk import WorkspaceClient
from pydantic import BaseModel, ConfigDict, ValidationError
from tenacity import retry, retry_if_result, stop_after_attempt, wait_random_exponential

from databricks_genai_inference.api.abstract.api_resource import APIResource
from databricks_genai_inference.api.abstract.foundation_model_object import FoundationModelObject
from databricks_genai_inference.api.exception import FoundationModelAPIException
from databricks_genai_inference.api.util import asend_request, is_internal_server_error, send_request

DATABRICKS_MODEL_URL_ENV = 'DATABRICKS_MODEL_URL'
DATABRICKS_HOST_ENV = 'DATABRICKS_HOST'
MODEL_URL_TEMPLATE = '{host}/serving-endpoints/databricks-{model}/invocations'


def get_url(
    host: str,
    model: str,
):
    """
    Returns the URL for the API.

    Args:
        host (str): The host for the API.
        model (str): The model for the API.

    Returns:
        The URL for the API.
    """

    env_url = os.getenv(DATABRICKS_MODEL_URL_ENV)
    if env_url:
        return env_url

    env_host = os.getenv(DATABRICKS_HOST_ENV)
    if env_host:
        host = env_host

    return MODEL_URL_TEMPLATE.format(host=host, model=model)


class FoundationModelAPIInput(BaseModel):
    """
    A class representing the input schema for a Foundation Model API request.

    Attributes:
        model (str): The name of the model to use.
        timeout (Optional[int]): The timeout for the API request.
    """
    model_config = ConfigDict(extra='forbid')

    model: str
    timeout: Optional[int] = None
    max_retries: Optional[int] = None


class FoundationModelAPIResource(APIResource):
    """
    A class representing a foundation model API resource.

    Attributes:
        SUPPORTED_MODEL_LIST (list): A list of supported models.
        DEFAULT_TIMEOUT (int): The default timeout for API requests.
        model_input (FoundationModelAPIInput): The input schema for the API.
        model_output (FoundationModelObject): The output schema for the API.
        model_streaming_output (FoundationModelObject): The streaming output schema for the API.
        config (Config): The configuration object for the API.
    """

    SUPPORTED_MODEL_LIST = []
    DEFAULT_TIMEOUT = 60
    MAX_RETRIES = 1
    model_input = FoundationModelAPIInput
    model_output = FoundationModelObject
    model_streaming_output = FoundationModelObject

    @classmethod
    def create(cls, client: requests.Session = None, **kwargs):
        """
        Creates a new API response.

        Args:
        **kwargs: The keyword arguments for the API.

        Returns:
        The result of the API query.
        """
        api_input = cls._parse_and_validate_request(**kwargs)
        return cls._make_query(client, api_input)

    @classmethod
    def _parse_and_validate_request(cls, **kwargs) -> FoundationModelAPIInput:
        """
        Parses and validates the request parameters.

        Args:
            **kwargs: The request parameters.

        Returns:
            A FoundationModelAPIInput object.

        Raises:
            FoundationModelAPIException: If the request parameters are invalid.
        """
        try:
            api_input = cls.model_input(**kwargs)
            if api_input.model not in cls.SUPPORTED_MODEL_LIST:
                raise FoundationModelAPIException(
                    message=f"Invalid model name: {api_input.model}. Supported model list: {cls.SUPPORTED_MODEL_LIST}")
            return api_input
        except ValidationError as e:
            raise FoundationModelAPIException(message=str(e)) from e

    @classmethod
    def _make_query(cls, client: requests.Session, model_input: FoundationModelAPIInput):
        """
        Makes a query to the API.

        Args:
        model_input (FoundationModelAPIInput): The input for the API.

        Returns:
        The response of the API query.

        Raises:
        FoundationModelAPIException: If the API query fails.
        """

        w = WorkspaceClient()
        url = get_url(host=w.config.host, model=model_input.model)
        headers = {
            'Content-Type': 'application/json',
            'X-Databricks-Endpoints-API-Client': 'Generative AI Inference (Mosaic) SDK'
        }
        headers = headers | w.config.authenticate()
        json = model_input.model_dump(exclude_unset=True)
        timeout = json.pop("timeout", cls.DEFAULT_TIMEOUT)
        max_retries = json.pop("max_retries", cls.MAX_RETRIES)
        try:
            if model_input.model_dump().get("stream", False):
                return cls._get_streaming_response(client=client,
                                                   url=url,
                                                   headers=headers,
                                                   json=json,
                                                   timeout=timeout,
                                                   max_retries=max_retries)
            else:
                return cls._get_non_streaming_response(client=client,
                                                       url=url,
                                                       headers=headers,
                                                       json=json,
                                                       timeout=timeout,
                                                       max_retries=max_retries)
        except requests.exceptions.ReadTimeout as e:
            raise FoundationModelAPIException(message=f'API request timed out after {timeout} seconds') from e
        except requests.exceptions.ConnectionError as e:
            raise FoundationModelAPIException(message="API request failed with connection error") from e
        except FoundationModelAPIException as e:
            raise e

    @classmethod
    def _get_non_streaming_response(cls, client, url, headers, json, timeout, max_retries):
        """
        Sends a request to the API and returns the non-streaming response.

        Args:
        url (str): The URL for the API.
        headers (dict): The headers for the API request.
        json (dict): The JSON data for the API request.
        timeout (int): The timeout for the API request.

        Raises:
        NotImplementedError: If the method is not implemented.
        """
        retry_req = retry(retry=retry_if_result(is_internal_server_error),
                          wait=wait_random_exponential(min=1, max=timeout),
                          stop=stop_after_attempt(max_retries),
                          retry_error_callback=lambda retry: retry.outcome.result())(send_request)
        response = retry_req(client=client, url=url, headers=headers, json=json, timeout=timeout)
        if response.ok:
            try:
                return cls.model_output(response.json())
            except requests.JSONDecodeError as e:
                raise FoundationModelAPIException(response=response, url=url) from e
        else:
            raise FoundationModelAPIException(response=response, url=url)

    @classmethod
    def _get_streaming_response(cls, client, url, headers, json, timeout, max_retries):
        """
        Sends a request to the API and returns the streaming response.

        Args:
        url (str): The URL for the API.
        headers (dict): The headers for the API request.
        json (dict): The JSON data for the API request.
        timeout (int): The timeout for the API request.

        Raises:
        NotImplementedError: If the method is not implemented.
        """
        retry_req = retry(retry=retry_if_result(is_internal_server_error),
                          wait=wait_random_exponential(min=1, max=timeout),
                          stop=stop_after_attempt(max_retries),
                          retry_error_callback=lambda retry: retry.outcome.result())(send_request)
        response = retry_req(url=url, headers=headers, json=json, timeout=timeout, client=client)
        if response:
            try:
                for line in response.iter_lines():
                    if line:
                        line = line.decode('utf-8')
                        if line.startswith("data: "):
                            line = line[len("data: "):]
                        if line == '[DONE]':
                            break
                        loaded_json = json_lib.loads(line)
                        if loaded_json:
                            yield cls.model_streaming_output(loaded_json)
            except json_lib.decoder.JSONDecodeError as e:
                raise FoundationModelAPIException(response=response, message="JSONDecodeError", url=url) from e
        else:
            raise FoundationModelAPIException(response=response, url=url)

    @classmethod
    async def acreate(cls, client: httpx.AsyncClient = None, **kwargs):
        """
        Creates a new API response.

        Args:
        client (httpx.AsyncClient): The client for http call.
        **kwargs: The keyword arguments for the API.

        Returns:
        The result of the API query.
        """
        model_input = cls._parse_and_validate_request(**kwargs)
        return await cls._amake_query(client, model_input)

    @classmethod
    async def _amake_query(cls, client: httpx.AsyncClient, model_input: FoundationModelAPIInput):
        """
        Makes a query to the API.

        Args:
        client (httpx.AsyncClient): The client for http call.
        model_input (FoundationModelAPIInput): The input for the API.

        Returns:
        The response of the API query.

        Raises:
        FoundationModelAPIException: If the API query fails.
        """
        w = WorkspaceClient()
        url = f'{w.config.host}/serving-endpoints/databricks-{model_input.model}/invocations'
        headers = {
            'Content-Type': 'application/json',
            'X-Databricks-Endpoints-API-Client': 'Generative AI Inference (Mosaic) SDK'
        }
        headers = w.config.authenticate() | headers
        json = model_input.model_dump(exclude_unset=True)
        timeout = json.pop("timeout", cls.DEFAULT_TIMEOUT)
        max_retries = json.pop("max_retries", cls.MAX_RETRIES)

        try:
            if model_input.model_dump().get("stream", False):
                return await cls._aget_streaming_response(client=client,
                                                          url=url,
                                                          headers=headers,
                                                          json=json,
                                                          timeout=timeout,
                                                          max_retries=max_retries)
            else:
                return await cls._aget_non_streaming_response(client=client,
                                                              url=url,
                                                              headers=headers,
                                                              json=json,
                                                              timeout=timeout,
                                                              max_retries=max_retries)
        except httpx.ReadTimeout as e:
            raise FoundationModelAPIException(message=f'API request timed out after {timeout} seconds') from e
        except httpx.ConnectError as e:
            raise FoundationModelAPIException(message="API request failed with connection error") from e
        except FoundationModelAPIException as e:
            raise e

    @classmethod
    async def _aget_non_streaming_response(cls, client, url, headers, json, timeout, max_retries):
        """
        Parse and returns the non-streaming response.

        Args:
        url (str): The URL for the API.
        response (httpx.Resonse): The response from the post request.
        """
        asend_request_with_retry = retry(retry=retry_if_result(is_internal_server_error),
                                         wait=wait_random_exponential(min=1, max=timeout),
                                         stop=stop_after_attempt(max_retries),
                                         retry_error_callback=lambda retry: retry.outcome.result())(asend_request)
        response = await asend_request_with_retry(client=client, url=url, headers=headers, json=json, timeout=timeout)
        if response.status_code < 400:
            try:
                response_body = response.json()
                return cls.model_output(response_body)
            except httpx.DecodingError as e:
                raise FoundationModelAPIException(url=url, response=response) from e
        else:
            raise FoundationModelAPIException(url=url, response=response)

    @classmethod
    async def _aget_streaming_response(cls, client, url, headers, json, timeout, max_retries):
        """
        Parse and returns the streaming response.

        Args:
        url (str): The URL for the API.
        response (httpx.Resonse): The response from the post request.
        """
        asend_request_with_retry = retry(retry=retry_if_result(is_internal_server_error),
                                         wait=wait_random_exponential(min=1, max=timeout),
                                         stop=stop_after_attempt(max_retries),
                                         retry_error_callback=lambda retry: retry.outcome.result())(asend_request)
        response = await asend_request_with_retry(client=client, url=url, headers=headers, json=json, timeout=timeout)
        return AsyncStreamResponse(url, response, cls.model_streaming_output)


class AsyncStreamResponse:
    """
    A class representing the async stream response, which works as a async iterator.
    """

    def __init__(self, url, response, model_streaming_output_cls):
        self._url = url
        self._response = response
        self._model_streaming_output_cls = model_streaming_output_cls
        self._closed = False
        self._iterator = self.__stream__()

    def __del__(self):
        if not self._closed:
            asyncio.run(self._response.aclose())

    async def __aiter__(self):
        try:
            async for item in self._iterator:
                yield item
        finally:
            await self._response.aclose()
            self._closed = True

    async def __stream__(self):
        if self._response.status_code < 400:
            try:
                async for line in self._response.aiter_lines():
                    if line:
                        if line.startswith("data: "):
                            line = line[len("data: "):]
                        if line == '[DONE]':
                            break
                        loaded_json = json_lib.loads(line)
                        if loaded_json:
                            yield self._model_streaming_output_cls(loaded_json)
            except json_lib.decoder.JSONDecodeError as e:
                raise FoundationModelAPIException(url=self._url, message="JSONDecodeError",
                                                  response=self._response) from e
        else:
            raise FoundationModelAPIException(url=self._url, response=self._response)



import unittest
from unittest.mock import patch

from constants import (CHAT_COMPLETION_MESSAGES, CHAT_COMPLETION_MODEL_NAME, CHAT_COMPLETION_MODEL_SYSTEM_MESSAGE,
                       CHAT_COMPLETION_RESPONSE_OBJECT_1, CHAT_SESSION_PARAMETERS, COMPLETION_PROMPT_1)

from databricks_genai_inference import ChatSession


class TestChatSession(unittest.TestCase):

    def setUp(self):
        self.model = CHAT_COMPLETION_MODEL_NAME
        self.system_message = CHAT_COMPLETION_MODEL_SYSTEM_MESSAGE
        self.chat_session = ChatSession(self.model, self.system_message, **CHAT_SESSION_PARAMETERS)

    @patch("databricks_genai_inference.ChatCompletion.create", return_value=CHAT_COMPLETION_RESPONSE_OBJECT_1)
    def test_reply(self, mocked_request):
        self.chat_session.reply(COMPLETION_PROMPT_1)
        mocked_request.assert_called_once_with(model=self.model,
                                               messages=CHAT_COMPLETION_MESSAGES,
                                               **CHAT_SESSION_PARAMETERS)
        self.assertEqual(self.chat_session.chat_history[-1]["role"], "assistant")
        self.assertEqual(self.chat_session.chat_history[-1]["content"], CHAT_COMPLETION_RESPONSE_OBJECT_1.message)

    @patch("databricks_genai_inference.ChatCompletion.create", return_value=CHAT_COMPLETION_RESPONSE_OBJECT_1)
    def test_last(self, mocked_request):
        self.assertEqual(self.chat_session.last, "")
        self.chat_session.reply(COMPLETION_PROMPT_1)
        self.assertEqual(self.chat_session.last, CHAT_COMPLETION_RESPONSE_OBJECT_1.message)

    @patch("databricks_genai_inference.ChatCompletion.create", return_value=CHAT_COMPLETION_RESPONSE_OBJECT_1)
    def test_history(self, mocked_request):
        self.assertEqual(self.chat_session.history, [{"role": "system", "content": self.system_message}])
        self.chat_session.reply(COMPLETION_PROMPT_1)
        self.assertEqual(self.chat_session.history, [
            {
                "role": "system",
                "content": self.system_message
            },
            {
                "role": "user",
                "content": COMPLETION_PROMPT_1
            },
            {
                "role": "assistant",
                "content": CHAT_COMPLETION_RESPONSE_OBJECT_1.message
            },
        ])

    @patch("databricks_genai_inference.ChatCompletion.create", return_value=CHAT_COMPLETION_RESPONSE_OBJECT_1)
    def test_pretty_history(self, mocked_request):
        self.assertEqual(self.chat_session.pretty_history, f"\nsystem: {self.system_message}")
        self.chat_session.reply(COMPLETION_PROMPT_1)
        self.assertEqual(
            self.chat_session.pretty_history,
            f"\nsystem: {self.system_message}\nuser: {COMPLETION_PROMPT_1}\nassistant: {CHAT_COMPLETION_RESPONSE_OBJECT_1.message}"
        )

    @patch("databricks_genai_inference.ChatCompletion.create", return_value=CHAT_COMPLETION_RESPONSE_OBJECT_1)
    def test_count(self, mocked_request):
        self.assertEqual(self.chat_session.count, 0)
        self.chat_session.reply(COMPLETION_PROMPT_1)
        self.assertEqual(self.chat_session.count, 1)



"""Abstract class for API resources.
"""
from abc import ABC, abstractmethod
from typing import AsyncGenerator, Generator, Union

from pydantic import BaseModel

from databricks_genai_inference.api.abstract.foundation_model_object import FoundationModelObject


class APIResource(ABC):
    """ Abstract class for API resources.
    """

    @classmethod
    @abstractmethod
    def create(cls, **kwargs) -> Union[FoundationModelObject, Generator[FoundationModelObject, None, None]]:
        pass

    @classmethod
    @abstractmethod
    def _parse_and_validate_request(cls, **kwargs) -> BaseModel:
        pass

    @classmethod
    @abstractmethod
    def _get_non_streaming_response(cls, client, url, headers, json, timeout) -> FoundationModelObject:
        pass

    @classmethod
    @abstractmethod
    def _get_streaming_response(cls, url, client, headers, json,
                                timeout) -> Generator[FoundationModelObject, None, None]:
        pass

    @classmethod
    @abstractmethod
    async def acreate(cls, **kwargs) -> Union[FoundationModelObject, AsyncGenerator[FoundationModelObject, None]]:
        pass

    @classmethod
    @abstractmethod
    async def _aget_non_streaming_response(cls, client, url, headers, json, timeout) -> FoundationModelObject:
        pass

    @classmethod
    @abstractmethod
    async def _aget_streaming_response(cls, client, url, headers, json,
                                       timeout) -> AsyncGenerator[FoundationModelObject, None]:
        pass



# Import the necessary modules
import types

import httpx
import pytest
from constants import (CHAT_COMPLETION_MESSAGES, CHAT_COMPLETION_MODEL_NAME, COMPLETION_MODEL_NAME, COMPLETION_PROMPT_1,
                       COMPLETION_PROMPT_2, EMBEDDING_INPUT_1, EMBEDDING_INPUT_2, EMBEDDING_INSTRUCTION,
                       EMBEDDING_MODEL_NAME, TEST_ECHO, TEST_ERROR_BEHAVIOR, TEST_MAX_RETRIES, TEST_MAX_TOKENS, TEST_N,
                       TEST_STOP, TEST_SUFFIX, TEST_TEMPERATURE, TEST_TIMEOUT, TEST_TOP_K, TEST_TOP_P,
                       TEST_USE_RAW_PROMPT, TEST_USER)

from databricks_genai_inference import (ChatCompletion, ChatCompletionChunkObject, ChatCompletionObject, Completion,
                                        CompletionChunkObject, CompletionObject, Embedding, EmbeddingObject)
from databricks_genai_inference.api.abstract.foundation_model_api_resource import AsyncStreamResponse


def test_embedding():
    kwargs = {
        "model": EMBEDDING_MODEL_NAME,
        "instruction": EMBEDDING_INSTRUCTION,
        "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
        "user": TEST_USER,
        "timeout": TEST_TIMEOUT,
        "max_retries": TEST_MAX_RETRIES,
    }
    try:
        response = Embedding.create(**kwargs)
        assert isinstance(response, EmbeddingObject)
    except Exception as e:
        assert False, f"Test failed due to an exception"


@pytest.mark.asyncio
async def test_async_embedding():
    kwargs = {
        "model": EMBEDDING_MODEL_NAME,
        "instruction": EMBEDDING_INSTRUCTION,
        "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
        "user": TEST_USER,
        "timeout": TEST_TIMEOUT,
        "max_retries": TEST_MAX_RETRIES,
    }
    try:
        async with httpx.AsyncClient() as client:
            response = await Embedding.acreate(client, **kwargs)

        assert isinstance(response, EmbeddingObject)
    except Exception as e:
        assert False, f"Test failed due to an exception: {e}"


def test_completion():
    kwargs = {
        "model": COMPLETION_MODEL_NAME,
        "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
        "temperature": TEST_TEMPERATURE,
        "stop": TEST_STOP,
        "max_tokens": TEST_MAX_TOKENS,
        "top_p": TEST_TOP_P,
        "top_k": TEST_TOP_K,
        "user": TEST_USER,
        "timeout": TEST_TIMEOUT,
        "n": TEST_N,
        "suffix": TEST_SUFFIX,
        "echo": TEST_ECHO,
        "error_behavior": TEST_ERROR_BEHAVIOR,
        "use_raw_prompt": TEST_USE_RAW_PROMPT,
        "max_retries": TEST_MAX_RETRIES,
    }
    try:
        response = Completion.create(**kwargs)
        assert isinstance(response, CompletionObject)
        response = Completion.create(stream=True, **kwargs)
        assert isinstance(response, types.GeneratorType)
    except Exception as e:
        assert False, f"Test failed due to an exception"


@pytest.mark.asyncio
async def test_async_completion():
    kwargs = {
        "model": COMPLETION_MODEL_NAME,
        "prompt": COMPLETION_PROMPT_1,
        "temperature": TEST_TEMPERATURE,
        "stop": TEST_STOP,
        "max_tokens": TEST_MAX_TOKENS,
        "top_p": TEST_TOP_P,
        "top_k": TEST_TOP_K,
        "user": TEST_USER,
        "timeout": TEST_TIMEOUT,
        "n": TEST_N,
        "suffix": TEST_SUFFIX,
        "echo": TEST_ECHO,
        "error_behavior": TEST_ERROR_BEHAVIOR,
        "use_raw_prompt": TEST_USE_RAW_PROMPT,
        "max_retries": TEST_MAX_RETRIES,
    }
    try:
        async with httpx.AsyncClient() as client:
            response = await Completion.acreate(client, **kwargs)
            assert isinstance(response, CompletionObject)
            response = await Completion.acreate(client, stream=True, **kwargs)
            assert isinstance(response, AsyncStreamResponse)
            async for data in response:
                assert isinstance(data, CompletionChunkObject)
    except Exception as e:
        assert False, f"Test failed due to an exception: {e}"


def test_chat_completion():
    kwargs = {
        "model": CHAT_COMPLETION_MODEL_NAME,
        "messages": CHAT_COMPLETION_MESSAGES,
        "temperature": TEST_TEMPERATURE,
        "stop": TEST_STOP,
        "max_tokens": TEST_MAX_TOKENS,
        "top_p": TEST_TOP_P,
        "top_k": TEST_TOP_K,
        "user": TEST_USER,
        "timeout": TEST_TIMEOUT,
        "n": TEST_N,
        "max_retries": TEST_MAX_RETRIES,
    }
    try:
        response = ChatCompletion.create(**kwargs)
        assert isinstance(response, ChatCompletionObject)
        response = ChatCompletion.create(stream=True, **kwargs)
        assert isinstance(response, types.GeneratorType)
    except Exception as e:
        assert False, f"Test failed due to an exception"


@pytest.mark.asyncio
async def test_async_chat_completion():
    kwargs = {
        "model": CHAT_COMPLETION_MODEL_NAME,
        "messages": CHAT_COMPLETION_MESSAGES,
        "temperature": TEST_TEMPERATURE,
        "stop": TEST_STOP,
        "max_tokens": TEST_MAX_TOKENS,
        "top_p": TEST_TOP_P,
        "top_k": TEST_TOP_K,
        "user": TEST_USER,
        "timeout": TEST_TIMEOUT,
        "n": TEST_N,
        "max_retries": TEST_MAX_RETRIES,
    }
    try:
        async with httpx.AsyncClient() as client:
            response = await ChatCompletion.acreate(client, **kwargs)
            assert isinstance(response, ChatCompletionObject)
            response = await ChatCompletion.acreate(client, stream=True, **kwargs)
            assert isinstance(response, AsyncStreamResponse)
            async for data in response:
                assert isinstance(data, ChatCompletionChunkObject)
    except Exception as e:
        assert False, f"Test failed due to an exception: {e}"


if __name__ == "__main__":
    pytest.main()



from unittest.mock import AsyncMock, patch

import pytest
from constants import (CHAT_COMPLETION_MESSAGES, CHAT_COMPLETION_MODEL_NAME, COMPLETION_MODEL_NAME, COMPLETION_PROMPT_1,
                       COMPLETION_PROMPT_2, EMBEDDING_INPUT_1, EMBEDDING_INPUT_2, EMBEDDING_INSTRUCTION,
                       EMBEDDING_MODEL_NAME, TEST_API_KEY, TEST_ECHO, TEST_ERROR_BEHAVIOR, TEST_HOST_NAME,
                       TEST_MAX_RETRIES, TEST_MAX_TOKENS, TEST_N, TEST_STOP, TEST_SUFFIX, TEST_TEMPERATURE,
                       TEST_TIMEOUT, TEST_TOP_K, TEST_TOP_P, TEST_USE_RAW_PROMPT, TEST_USER)

from databricks_genai_inference import ChatCompletion, Completion, Embedding


class TestAPIRequest:

    def _get_expected_header(self, test_api_key: str):
        return {
            'Authorization': "Bearer " + test_api_key,
            'Content-Type': 'application/json',
            'X-Databricks-Endpoints-API-Client': 'Generative AI Inference (Mosaic) SDK'
        }

    # Fixture to change an environment variable temporarily
    @pytest.fixture(autouse=True)
    def mock_env_var(self, monkeypatch):
        monkeypatch.setenv('DATABRICKS_HOST', TEST_HOST_NAME)
        monkeypatch.setenv('DATABRICKS_TOKEN', TEST_API_KEY)

    @patch('databricks_genai_inference.ChatCompletion._get_non_streaming_response')
    def test_chat_completion_request_non_streaming(self, mocked_request):
        kwargs = {
            "model": CHAT_COMPLETION_MODEL_NAME,
            "messages": CHAT_COMPLETION_MESSAGES,
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{CHAT_COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": CHAT_COMPLETION_MODEL_NAME,
                "messages": CHAT_COMPLETION_MESSAGES,
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        ChatCompletion.create(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @pytest.mark.asyncio
    @patch('databricks_genai_inference.ChatCompletion._aget_non_streaming_response', new_callable=AsyncMock)
    async def test_chat_completion_async_request_non_streaming(self, mocked_request):
        kwargs = {
            "model": CHAT_COMPLETION_MODEL_NAME,
            "messages": CHAT_COMPLETION_MESSAGES,
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{CHAT_COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": CHAT_COMPLETION_MODEL_NAME,
                "messages": CHAT_COMPLETION_MESSAGES,
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }

        await ChatCompletion.acreate(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @patch('databricks_genai_inference.ChatCompletion._get_streaming_response')
    def test_chat_completion_request_streaming(self, mocked_request):
        kwargs = {
            "model": CHAT_COMPLETION_MODEL_NAME,
            "messages": CHAT_COMPLETION_MESSAGES,
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "stream": True,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{CHAT_COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": CHAT_COMPLETION_MODEL_NAME,
                "messages": CHAT_COMPLETION_MESSAGES,
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
                "stream": True
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        ChatCompletion.create(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @pytest.mark.asyncio
    @patch('databricks_genai_inference.ChatCompletion._aget_streaming_response', new_callable=AsyncMock)
    async def test_chat_completion_async_request_streaming(self, mocked_request):
        kwargs = {
            "model": CHAT_COMPLETION_MODEL_NAME,
            "messages": CHAT_COMPLETION_MESSAGES,
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "stream": True,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{CHAT_COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": CHAT_COMPLETION_MODEL_NAME,
                "messages": CHAT_COMPLETION_MESSAGES,
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
                "stream": True
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        await ChatCompletion.acreate(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @patch('databricks_genai_inference.Completion._get_non_streaming_response')
    def test_completion_request_non_streaming(self, mocked_request):
        kwargs = {
            "model": COMPLETION_MODEL_NAME,
            "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "suffix": TEST_SUFFIX,
            "echo": TEST_ECHO,
            "error_behavior": TEST_ERROR_BEHAVIOR,
            "use_raw_prompt": TEST_USE_RAW_PROMPT,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": COMPLETION_MODEL_NAME,
                "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
                "suffix": TEST_SUFFIX,
                "echo": TEST_ECHO,
                "error_behavior": TEST_ERROR_BEHAVIOR,
                "use_raw_prompt": TEST_USE_RAW_PROMPT,
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        Completion.create(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @pytest.mark.asyncio
    @patch('databricks_genai_inference.Completion._aget_non_streaming_response', new_callable=AsyncMock)
    async def test_completion_async_request_non_streaming(self, mocked_request):
        kwargs = {
            "model": COMPLETION_MODEL_NAME,
            "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "suffix": TEST_SUFFIX,
            "echo": TEST_ECHO,
            "error_behavior": TEST_ERROR_BEHAVIOR,
            "use_raw_prompt": TEST_USE_RAW_PROMPT,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": COMPLETION_MODEL_NAME,
                "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
                "suffix": TEST_SUFFIX,
                "echo": TEST_ECHO,
                "error_behavior": TEST_ERROR_BEHAVIOR,
                "use_raw_prompt": TEST_USE_RAW_PROMPT,
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        await Completion.acreate(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @patch('databricks_genai_inference.Completion._get_streaming_response')
    def test_completion_request_streaming(self, mocked_request):
        kwargs = {
            "model": COMPLETION_MODEL_NAME,
            "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "suffix": TEST_SUFFIX,
            "echo": TEST_ECHO,
            "error_behavior": TEST_ERROR_BEHAVIOR,
            "use_raw_prompt": TEST_USE_RAW_PROMPT,
            "stream": True,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": COMPLETION_MODEL_NAME,
                "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
                "suffix": TEST_SUFFIX,
                "echo": TEST_ECHO,
                "error_behavior": TEST_ERROR_BEHAVIOR,
                "use_raw_prompt": TEST_USE_RAW_PROMPT,
                "stream": True
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        Completion.create(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @pytest.mark.asyncio
    @patch('databricks_genai_inference.Completion._aget_streaming_response', new_callable=AsyncMock)
    async def test_completion_async_request_streaming(self, mocked_request):
        kwargs = {
            "model": COMPLETION_MODEL_NAME,
            "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "suffix": TEST_SUFFIX,
            "echo": TEST_ECHO,
            "error_behavior": TEST_ERROR_BEHAVIOR,
            "use_raw_prompt": TEST_USE_RAW_PROMPT,
            "stream": True,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{COMPLETION_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": COMPLETION_MODEL_NAME,
                "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
                "temperature": TEST_TEMPERATURE,
                "stop": TEST_STOP,
                "max_tokens": TEST_MAX_TOKENS,
                "top_p": TEST_TOP_P,
                "top_k": TEST_TOP_K,
                "user": TEST_USER,
                "n": TEST_N,
                "suffix": TEST_SUFFIX,
                "echo": TEST_ECHO,
                "error_behavior": TEST_ERROR_BEHAVIOR,
                "use_raw_prompt": TEST_USE_RAW_PROMPT,
                "stream": True
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        await Completion.acreate(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @patch('databricks_genai_inference.Embedding._get_non_streaming_response')
    def test_embedding_request_non_streaming(self, mocked_request):
        kwargs = {
            "model": EMBEDDING_MODEL_NAME,
            "instruction": EMBEDDING_INSTRUCTION,
            "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{EMBEDDING_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": EMBEDDING_MODEL_NAME,
                "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
                "instruction": EMBEDDING_INSTRUCTION,
                "user": TEST_USER,
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        Embedding.create(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)

    @pytest.mark.asyncio
    @patch('databricks_genai_inference.Embedding._aget_non_streaming_response', new_callable=AsyncMock)
    async def test_embedding_async_request_non_streaming(self, mocked_request):
        kwargs = {
            "model": EMBEDDING_MODEL_NAME,
            "instruction": EMBEDDING_INSTRUCTION,
            "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        expected_request = {
            "client": None,
            "url": f'{TEST_HOST_NAME}/serving-endpoints/databricks-{EMBEDDING_MODEL_NAME}/invocations',
            "headers": self._get_expected_header(TEST_API_KEY),
            "json": {
                "model": EMBEDDING_MODEL_NAME,
                "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
                "instruction": EMBEDDING_INSTRUCTION,
                "user": TEST_USER,
            },
            "timeout": TEST_TIMEOUT,
            "max_retries": TEST_MAX_RETRIES,
        }
        await Embedding.acreate(**kwargs)
        mocked_request.assert_called_once_with(**expected_request)



from unittest.mock import patch

import pytest
from constants import (CHAT_COMPLETION_MESSAGES, CHAT_COMPLETION_MODEL_NAME, COMPLETION_MODEL_NAME, COMPLETION_PROMPT_1,
                       COMPLETION_PROMPT_2, EMBEDDING_INPUT_1, EMBEDDING_INPUT_2, EMBEDDING_INSTRUCTION,
                       EMBEDDING_MODEL_NAME, FAKE_ARG_VALUE, TEST_ECHO, TEST_ERROR_BEHAVIOR, TEST_MAX_TOKENS, TEST_N,
                       TEST_STOP, TEST_SUFFIX, TEST_TEMPERATURE, TEST_TIMEOUT, TEST_TOP_K, TEST_TOP_P,
                       TEST_USE_RAW_PROMPT, TEST_USER)

from databricks_genai_inference import ChatCompletion, Completion, Embedding
from databricks_genai_inference.api.chat_completion import ChatCompletionAPIInput
from databricks_genai_inference.api.completion import CompletionAPIInput
from databricks_genai_inference.api.embedding import EmbeddingAPIInput
from databricks_genai_inference.api.exception import FoundationModelAPIException


class TestInputValidation:

    @patch('databricks_genai_inference.ChatCompletion._make_query')
    def test_chat_completion_correct_input(self, mocked_request):
        kwargs = {
            "model": CHAT_COMPLETION_MODEL_NAME,
            "messages": CHAT_COMPLETION_MESSAGES,
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
        }
        expected_model_input = ChatCompletionAPIInput(
            model=CHAT_COMPLETION_MODEL_NAME,
            messages=CHAT_COMPLETION_MESSAGES,
            temperature=TEST_TEMPERATURE,
            stop=TEST_STOP,
            max_tokens=TEST_MAX_TOKENS,
            top_p=TEST_TOP_P,
            top_k=TEST_TOP_K,
            user=TEST_USER,
            timeout=TEST_TIMEOUT,
            n=TEST_N,
        )
        ChatCompletion.create(**kwargs)
        mocked_request.assert_called_once_with(None, expected_model_input)

    @patch('databricks_genai_inference.ChatCompletion._make_query')
    def test_chat_completion_correct_input_required(self, mocked_request):
        kwargs = {
            "model": CHAT_COMPLETION_MODEL_NAME,
            "messages": CHAT_COMPLETION_MESSAGES,
        }
        expected_model_input = ChatCompletionAPIInput(
            model=CHAT_COMPLETION_MODEL_NAME,
            messages=CHAT_COMPLETION_MESSAGES,
        )
        ChatCompletion.create(**kwargs)
        mocked_request.assert_called_once_with(None, expected_model_input)

    @patch('databricks_genai_inference.ChatCompletion._make_query')
    def test_chat_completion_wrong_input(self, mocked_request):
        with pytest.raises(FoundationModelAPIException) as error:
            ChatCompletion.create(messages=CHAT_COMPLETION_MESSAGES)
        assert "validation error" in str(error.value), "missing model name should raise validation error"

        with pytest.raises(FoundationModelAPIException) as error:
            ChatCompletion.create(model=CHAT_COMPLETION_MODEL_NAME + "-fake")
        assert "validation error" in str(error.value), "wrong model name should raise validation error"

        with pytest.raises(FoundationModelAPIException) as error:
            ChatCompletion.create(FAKE_ARG_NAME=FAKE_ARG_VALUE)
        assert "validation error" in str(error.value), "unrecognized should raise validation error"

    @patch('databricks_genai_inference.Completion._make_query')
    def test_completion_correct_input(self, mocked_request):
        kwargs = {
            "model": COMPLETION_MODEL_NAME,
            "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
            "temperature": TEST_TEMPERATURE,
            "stop": TEST_STOP,
            "max_tokens": TEST_MAX_TOKENS,
            "top_p": TEST_TOP_P,
            "top_k": TEST_TOP_K,
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
            "n": TEST_N,
            "suffix": TEST_SUFFIX,
            "echo": TEST_ECHO,
            "error_behavior": TEST_ERROR_BEHAVIOR,
            "use_raw_prompt": TEST_USE_RAW_PROMPT,
        }
        expected_model_input = CompletionAPIInput(
            model=COMPLETION_MODEL_NAME,
            prompt=[COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
            temperature=TEST_TEMPERATURE,
            stop=TEST_STOP,
            max_tokens=TEST_MAX_TOKENS,
            top_p=TEST_TOP_P,
            top_k=TEST_TOP_K,
            user=TEST_USER,
            timeout=TEST_TIMEOUT,
            n=TEST_N,
            suffix=TEST_SUFFIX,
            echo=TEST_ECHO,
            error_behavior=TEST_ERROR_BEHAVIOR,
            use_raw_prompt=TEST_USE_RAW_PROMPT,
        )
        Completion.create(**kwargs)
        mocked_request.assert_called_once_with(None, expected_model_input)

    @patch('databricks_genai_inference.Completion._make_query')
    def test_completion_correct_input_required(self, mocked_request):
        kwargs = {
            "model": COMPLETION_MODEL_NAME,
            "prompt": [COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
        }
        expected_model_input = CompletionAPIInput(
            model=COMPLETION_MODEL_NAME,
            prompt=[COMPLETION_PROMPT_1, COMPLETION_PROMPT_2],
        )
        Completion.create(**kwargs)
        mocked_request.assert_called_once_with(None, expected_model_input)

    @patch('databricks_genai_inference.Completion._make_query')
    def test_completion_wrong_input(self, mocked_request):
        with pytest.raises(FoundationModelAPIException) as error:
            Completion.create(prompt=[COMPLETION_PROMPT_1, COMPLETION_PROMPT_2])
        assert "validation error" in str(error.value), "missing model name should raise validation error"

        with pytest.raises(FoundationModelAPIException) as error:
            Completion.create(model=COMPLETION_MODEL_NAME + "-fake")
        assert "validation error" in str(error.value), "wrong model name should raise validation error"

        with pytest.raises(FoundationModelAPIException) as error:
            Completion.create(FAKE_ARG_NAME=FAKE_ARG_VALUE)
        assert "validation error" in str(error.value), "unrecognized should raise validation error"

    @patch('databricks_genai_inference.Embedding._make_query')
    def test_embedding_correct_input(self, mocked_request):
        kwargs = {
            "model": EMBEDDING_MODEL_NAME,
            "instruction": EMBEDDING_INSTRUCTION,
            "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
            "user": TEST_USER,
            "timeout": TEST_TIMEOUT,
        }
        expected_model_input = EmbeddingAPIInput(
            model=EMBEDDING_MODEL_NAME,
            instruction=EMBEDDING_INSTRUCTION,
            input=[EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
            user=TEST_USER,
            timeout=TEST_TIMEOUT,
        )
        Embedding.create(**kwargs)
        mocked_request.assert_called_once_with(None, expected_model_input)

    @patch('databricks_genai_inference.Embedding._make_query')
    def test_embedding_correct_input_required(self, mocked_request):
        kwargs = {
            "model": EMBEDDING_MODEL_NAME,
            "input": [EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
        }
        expected_model_input = EmbeddingAPIInput(
            model=EMBEDDING_MODEL_NAME,
            input=[EMBEDDING_INPUT_1, EMBEDDING_INPUT_2],
        )
        Embedding.create(**kwargs)
        mocked_request.assert_called_once_with(None, expected_model_input)

    @patch('databricks_genai_inference.Embedding._make_query')
    def test_embedding_wrong_input(self, mocked_request):
        with pytest.raises(FoundationModelAPIException) as error:
            Embedding.create(input=[EMBEDDING_INPUT_1, EMBEDDING_INPUT_2])
        assert "validation error" in str(error.value), "missing model name should raise validation error"

        with pytest.raises(FoundationModelAPIException) as error:
            Embedding.create(model=EMBEDDING_MODEL_NAME + "-fake")
        assert "validation error" in str(error.value), "wrong model name should raise validation error"

        with pytest.raises(FoundationModelAPIException) as error:
            Embedding.create(FAKE_ARG_NAME=FAKE_ARG_VALUE)
        assert "validation error" in str(error.value), "unrecognized should raise validation error"

        with pytest.raises(FoundationModelAPIException) as error:
            Embedding.create(model=EMBEDDING_MODEL_NAME, input=EMBEDDING_INPUT_1, stream=True)
        assert "validation error" in str(error.value), "embedding model does not support streaming"
